/// Define how to parse response json
/// If you want to support more, PR is welcome
enum Parser {
  /// Each model class must provide 'factory T.fromJson(Map<String, dynamic> json)'
  /// For more detail, please visit 'https://github.com/trevorwang/retrofit.dart#type-conversion'
  JsonSerializable,

  /// same as [JsonSerializable] but with toMap and fromMap functions.
  MapSerializable,

  /// Each model class must add annotation '@jsonSerializable'
  /// For more detail, please visit 'https://github.com/k-paxian/dart-json-mapper'
  DartJsonMapper,

  /// Parse on a separate isolate using `compute` (Flutter only).
  ///
  /// Each model class must define a top-level function, taking the form
  /// ```
  /// FutureOr<T> deserializeT(Map<String, dynamic> json);
  /// FutureOr<dynamic> serializeTask(T object);
  /// ```
  ///
  /// If you want to handle lists of objects, either as return types or parameters, you should provide List counterparts.
  ///
  /// ```
  /// FutureOr<List<T>> deserializeTList(Map<String, dynamic> json);
  /// FutureOr<dynamic> serializeTList(List<T> objects);
  /// ```
  ///
  /// E.g.
  /// ----
  /// _In file user.dart_
  /// ```
  /// User deserializeUser(Map<String, dynamic> json) => User.fromJson(json);
  /// List<User> deserializeUserList(List<Map<String, dynamic>> json) =>
  ///     json.map((e) => User.fromJson(e)).toList();
  /// Map<String, dynamic> serializeUser(User object) => object.toJson();
  /// List<Map<String, dynamic>> serializeUserList(List<User> objects) =>
  ///     objects.map((e) => e.toJson()).toList();
  ///
  /// @JsonSerializable()
  /// class User {
  ///   factory User.fromJson(Map<String, dynamic> json) => _$UserFromJson(json);
  ///   Map<String, dynamic> toJson() => _$UserToJson(this);
  /// }
  /// ```
  FlutterCompute,

  SingleDynamicMapper,
}

abstract class CustomSerializer {
  Map<String, dynamic> serialize<T>(T value);
}

abstract class CustomDeserializer {
  T deserialize<T>(String name, Map<String, dynamic> value);
}
